/***************************************************************************
*   Copyright (C) 2004 by Kita Developers                                 *
*   ikemo@users.sourceforge.jp                                            *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
***************************************************************************/
#ifndef KITABOARDVIEW_H
#define KITABOARDVIEW_H

#include <kurl.h>

#include "threadlistview.h"

class QTreeWidgetItem;
class QWidget;

class KListViewItem;

namespace Kita {
    class BoardTabWidget;

    /**
     *
     * @author Hideki Ikemoto <ikemo@users.sourceforge.jp>
     */
    class BoardView : public ThreadListView
    {
        Q_OBJECT

        BoardTabWidget* m_parent;

    public:
        explicit BoardView(BoardTabWidget* parent);
        void init();
        const KUrl boardUrl();
        void toggleShowOldLogs();

    public slots:
        virtual void setFocus();
        void slotFocusSearchCombo();
        void reloadSubject();
        void loadBoard(const KUrl& url, bool online = true);
        void setFont(const QFont& font);
        void slotUpdateSubject(const KUrl& url);

    private:
        KUrl m_boardUrl;
        int m_unreadNum, m_readNum, m_newNum;
        bool m_showOldLogs;
        bool m_enableSizeChange;

        void updateKindLabel();
        void deleteLog(const KUrl& url);
        void loadLayout();
        void updateRow(int row, const KUrl& datUrl,
            const QDateTime& current, int id, int order);
        void saveHeaderOnOff();
        void loadHeaderOnOff();
        bool autoResize();
        void setAutoResize(bool flag);
        BoardView(const BoardView&);
        BoardView& operator=(const BoardView&);

    private slots:
        void slotCloseButton();
        void slotSizeChange(int section, int oldSize, int newSize);

    signals:
        void loadBoardCompleted(const KUrl&);
    };
}

#endif
