/***************************************************************************
*   Copyright (C) 2003 by Hideki Ikemoto                                  *
*   ikemo@users.sourceforge.jp                                            *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
***************************************************************************/

#include "favoritelistview.h"

#include <QtCore/QDateTime>
#include <QtGui/QApplication>
#include <QtGui/QClipboard>
#include <QtGui/QTableWidgetItem>

#include <kaction.h>
#include <kmenu.h>
#include <krun.h>

#include "threadlistviewitem.h"
#include "viewmediator.h"
#include "libkita/boarddatabase.h"
#include "libkita/datmanager.h"
#include "libkita/favoritethreads.h"
#include "libkita/kita_misc.h"
#include "libkita/thread.h"

using namespace Kita;

/**
 *
 */
FavoriteListView::FavoriteListView(QWidget* parent) : ThreadListView(parent)
{
    kindLabel->hide();

    connect(subjectList, SIGNAL(itemClicked(QTableWidgetItem*)),
             SLOT(loadThread(QTableWidgetItem*)));
    connect(reloadButton, SIGNAL(clicked()),
             SLOT(reload()));

    showColumn(ColumnBoard);
}

/**
 *
 */
void FavoriteListView::refresh()
{
    // clear
    m_hitList.clear();
    m_nextHitIndex = 0;
    m_prevquery = QStringList();

    subjectList->clearContents();

    subjectList->setSortingEnabled(false);
    int count = FavoriteThreads::count();
    subjectList->setRowCount(count);
    for (int i = 0; i < count; i++) {
        for (int j = 0, k = subjectList->columnCount(); j < k; j++) {
            ThreadListViewItem* item = new ThreadListViewItem(j);
            subjectList->setItem(i, j, item);
        }
    }
    subjectList->setSortingEnabled(true);
    // insert item.
    for (int i = 0; i < count; i++) {
        QString datUrl = FavoriteThreads::getDatUrl(i);

        QDateTime since;
        since.setTime_t(datToSince(datUrl));

        DatManager datManager(datUrl);
        int viewPos = datManager.getViewPos();
        int resNum = datManager.getResNum();

        BoardDatabase db(datUrl);
        subjectList->item(i, ColumnBoard)->setText(db.boardName());
        subjectList->item(i, ColumnSubject)
            ->setText(datManager.threadName());
        subjectList->item(i, ColumnReadNum)
            ->setText(QString("%1").arg(viewPos, 4));
        if (resNum > 0) {
            subjectList->item(i, ColumnResNum)
                ->setText(QString("%1").arg(resNum, 4));
        }
        if (resNum != 0 && resNum != viewPos) {
            subjectList->item(i, ColumnUnread)
                ->setText(QString("%1").arg(resNum - viewPos, 4));
        }
        subjectList->item(i, ColumnSince)
            ->setText(since.toString("yy/MM/dd hh:mm"));
        subjectList->item(i, ColumnDatUrl)->setText(datUrl);
    }
    subjectList->sortItems(ColumnBoard);
    for (int i = 0, j = subjectList->columnCount(); i < j; i++) {
        subjectList->resizeColumnToContents(i);
    }
    for (int i = 0, j = subjectList->rowCount(); i < j; i++) {
        subjectList->resizeRowToContents(i);
    }
}

/**
 *
 */
void FavoriteListView::loadThread(QTableWidgetItem* item)
{
    if (! item) return ;

    QString itemUrl = subjectList->item(item->row(), ColumnDatUrl)->text();

    for (int i = 0; FavoriteThreads::count() > i; i++) {
        QString datUrl = FavoriteThreads::getDatUrl(i);

        if (datUrl == itemUrl) {
            ViewMediator::getInstance()->openThread(datUrl);
        }
    }
}

/**
 *
 */
void FavoriteListView::reload()
{
    QList<QString> boardList;

    for (int i = 0; FavoriteThreads::count() > i; i++) {
        QString datUrl = FavoriteThreads::getDatUrl(i);
        BoardDatabase db(datUrl);
        QString boardUrl = db.boardUrl();
        if (boardList.contains(boardUrl) == 0) {
            boardList.append(boardUrl);
        }
    }

    QList<QString>::const_iterator it;
    for (it = boardList.begin(); it != boardList.end(); ++it) {
        bool online = true;
        QList<Thread*> threadList;
        QList<Thread*> tmpList;
        BoardDatabase db((*it));
        db.getThreadList(false, online, threadList, tmpList);
    }
}
