/***************************************************************************
*   Copyright (C) 2004 by Kita Developers                                 *
*   ikemo@users.sourceforge.jp                                            *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
***************************************************************************/
#ifndef KITAHTMLPART_H
#define KITAHTMLPART_H

#include <QtCore/QEvent>

#include <khtml_part.h>

/* mode */
enum {
    /* on main window */
    HTMLPART_MODE_MAINPART,

    /* on popup       */
    HTMLPART_MODE_POPUP
};

/* ID of user defined event */
const int EVENT_GotoAnchor = QEvent::User + 100;

class KUrl;

namespace DOM
{
    class Node;
}

namespace Kita
{
    class DomTree;
    class ResPopup;

    class HTMLPart : public KHTMLPart
    {
        Q_OBJECT

        DomTree* m_domtree;
        ResPopup* m_popup;

        /* basic information */
        int m_mode;
        KUrl m_datUrl;
        bool m_updatedKokoyon;


        /* cache */
        bool m_firstReceive;
        int m_centerNum;

        /* goto anchor */
        QStringList m_anchorStack;
        int m_jumpNumAfterLoading; /* see also slotFinishLoad() */

        /* mouse event */
        bool m_pushctrl;
        bool m_pushmidbt;
        bool m_pushrightbt;

        /* search */
        DOM::Node m_findNode;
        int m_findPos;
        int m_find_y;

        /* res popup */
        bool m_multiPopup;

    public:

        HTMLPart(QWidget* parent);
        ~HTMLPart();
        bool setup(int mode , const KUrl& url);

        /* rendering */
        void showResponses(int startnum, int endnum);
        void parseResponses(int startnum, int endnum);
        void showAll();
        void updateScreen(bool showHeaderEtc, bool clock);
        void setInnerHTML(const QString& innerHTML);

        /* cache */
        bool load(int centerNum);
        bool reload(int jumpNum);

        /* goto anchor */
        bool gotoAnchor(const QString& anc, bool pushPosition);

        /* search */
        bool findText(const QString &query, bool reverse);

        /* popup */
        bool isPopupVisible();

    public slots:

        /* rendering */
        void slotSetFaceOfHTMLPart();

        /* cache */
        void slotReceiveData();
        void slotFinishLoad();

        /* goto anchor */
        void slotGotoKokoyon();
        void slotGobackAnchor();

        /* Tugi100, Mae100 etc.  */
        void slotClickGotoFooter();

        /* res popup */
        void slotDeletePopup();
        void slotShowResPopup(const QPoint& point, int refNum, int refNum2);

    private:

        void clearPart();
        void redrawHTMLPart(const KUrl& datUrl, bool force);

        /* setup */
        void connectSignals();
        void createHTMLDocument();

        /* goto anchor */
        void pushCurrentPosition();
        QString getCurrentIdOfNode();

        /* popup menu */
        void showPopupMenu(const KUrl& kurl);

        /* click */
        void clickAnchor(const KUrl& urlin);
        void showWritePopupMenu(const QString& refstr);
        void showIdPopup(const QString& refstr);
        void showBePopupMenu(const QString& refstr);

        /* search */
        void findTextInit();

        /* res popup */
        void showPopup(const KUrl& url, const QString& innerHTML);
        void showPopupCore(const KUrl& url, const QString& innerHTML,
                const QPoint& point);
        bool startMultiPopup();
        bool isMultiPopupMode();
        void hidePopup();
        bool isUnderMouse(int mrgwd, int mrght);
        bool showSelectedDigitPopup();

        HTMLPart(const HTMLPart&);
        HTMLPart& operator=(const HTMLPart&);

    protected:

        /* user event */
        virtual void customEvent(QEvent * e);

        /* mouse event */
        virtual void khtmlMousePressEvent(khtml::MousePressEvent* e);


    private slots:

        /* click */
        void slotOpenUrlRequest(const KUrl&, const KParts::OpenUrlArguments&);

        /* res popup */
        void slotLeave();
        void slotVSliderReleased();
        void slotHSliderReleased();
        void slotHideChildPopup();
        void slotOnUrl(const QString& url);


    signals:
        void mousePressed(); /* to ThreadView */

        /* cache */
        void receiveData();
        void finishReload();

        /* res popup */
        void hideChildPopup();
    };

    /* user defined events */
    class GotoAnchorEvent : public QEvent
    {
        QString m_anc;

    public:

        GotoAnchorEvent(const QString& anc)
                : QEvent(QEvent::Type(EVENT_GotoAnchor)), m_anc(anc) {};

        const QString& getAnc() const { return m_anc; }
    };
}

#endif
