/***************************************************************************
*   Copyright (C) 2003 by Hideki Ikemoto, 2004 by 421                     *
*   ikemo@users.sourceforge.jp                                            *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
***************************************************************************/

#ifndef KITATABWIDGETBASE_H
#define KITATABWIDGETBASE_H

#include <QtCore/QEvent>

#include <ktabwidget.h>
#include <kxmlguiclient.h>
#include <kparts/part.h>
#include <kparts/partmanager.h>

class QIcon;

class KMenu;
class KUrl;

namespace KParts
{
    class DockMainWindow3;
}

/* ID of user defined event */
const int EVENT_CloseTab = QEvent::User + 100;
const int EVENT_ShowDock = QEvent::User + 101;
const int EVENT_FitImageToWinEvent = QEvent::User + 102;
const int EVENT_CancelMoszicEvent = QEvent::User + 103;

namespace Kita
{
    class KDE_EXPORT TabWidgetBase : public KTabWidget, public KXMLGUIClient
    {
        Q_OBJECT

    protected:
        KParts::PartManager* m_manager;


    public:
        explicit TabWidgetBase(QWidget* parent = 0);
        virtual ~TabWidgetBase();

    public slots:
        void slotCurrentChanged(QWidget* w);
        void slotCloseTab(int num);


    protected:
        virtual void customEvent(QEvent * e);
        virtual void deleteWidget(QWidget* w);
        KParts::Part* findPartFromWidget(QWidget* w);

        /*------------------------------------*/
        /* common tab actions */

    private:
        void setupActions();
        TabWidgetBase(const TabWidgetBase&);
        TabWidgetBase& operator=(const TabWidgetBase&);

    public slots:
        void slotConfigureKeys();
        void slotPrevTab();
        void slotNextTab();
        void slotCloseCurrentTab();
        void slotCloseOtherTab(int idx = -1);
        void slotCloseRightTab(int idx = -1);
        void slotCloseLeftTab(int idx = -1);
        void slotCloseAllTab();
    };

    /* user defined events */
    class CloseTabEvent : public QEvent
    {
        int m_pageindex;

    public:

        CloseTabEvent(int idx)
                : QEvent(QEvent::Type(EVENT_CloseTab)), m_pageindex(idx) {}

        int getIndex() const { return m_pageindex; }
    };


    class ShowDockEvent : public QEvent
    {
        bool m_activate;
        bool m_force;

    public:
        ShowDockEvent(bool activate, bool force) : QEvent(QEvent::Type(EVENT_ShowDock))
        , m_activate(activate), m_force(force) {}

        bool getActivate() const { return m_activate; }
        bool getForce() const { return m_force; }
    };
}

#endif
