/***************************************************************************
*   Copyright (C) 2004 by Kita Developers                                 *
*   ikemo@users.sourceforge.jp                                            *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
***************************************************************************/
#include "account.h"

#include <QtCore/QByteArray>
#include <QtCore/QRegExp>

#include <kurl.h>
#include <kio/job.h>
#include <kio/jobuidelegate.h>
#include <kio/slaveconfig.h>

#include "config.h"

using namespace Kita;

Account* Account::instance = 0;

Account* Account::getInstance()
{
    if (instance == 0) {
        instance = new Account();
    }
    return instance;
}

Account::Account()
        : m_isLogged(false)
{}

Account::~Account()
{}

bool Account::login(const QString& userID, const QString& password)
{
    return getInstance() ->loginInternal(userID, password);
}

bool Account::loginInternal(const QString& userID, const QString& password)
{
    m_data.clear();

    KUrl url("https://2chv.tora3.net/futen.cgi");
    QString postData = QString("ID=") + userID + QString("&PW=") + password;

    KIO::SlaveConfig::self() ->setConfigData("https",
            url.host(),
            "UserAgent",
            "DOLIB/1.00");
    m_job = KIO::http_post(url, postData.toUtf8(), false);

    connect(m_job, SIGNAL(data(KIO::Job*, const QByteArray&)),
             SLOT(slotReceiveData(KIO::Job*, const QByteArray&)));
    connect(m_job, SIGNAL(result(KIO::Job*)), SLOT(slotResult(KIO::Job*)));
    m_job->addMetaData("customHTTPHeader",
                        QString("X-2ch-UA: Kita/%1").arg(VERSION));
    m_job->addMetaData("content-type",
                        "Content-Type: application/x-www-form-urlencoded");

    QEventLoop m_eventLoop;
    m_eventLoop.exec();

    return m_isLogged;
}

void Account::slotReceiveData(KIO::Job*, const QByteArray& data)
{
    m_data += data;
}

void Account::slotResult(KIO::Job* job)
{
    m_job = 0;
    if (job->error()) {
        job->ui()->setWindow(0);
        job->ui()->showErrorMessage();
    }

    QString str(m_data);
    QRegExp regexp("SESSION-ID=(.*)");
    if (regexp.indexIn(str) == -1) {
        m_sessionID.clear();
        m_isLogged = false;
    } else {
        QString value = regexp.cap(1);

        QRegExp error("^ERROR:p+$");
        if (error.indexIn(value) == -1) {
            m_isLogged = true;
            m_sessionID = value;
        } else {
            m_isLogged = false;
            m_sessionID.clear();
        }
    }
    m_eventLoop.exit();
}
