/***************************************************************************
*   Copyright (C) 2004 by Kita Developers                                 *
*   ikemo@users.sourceforge.jp                                            *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
***************************************************************************/

#include "boarddata.h"

using namespace Kita;

/*---------------------------------------------------------*/

/* BoardData */

BoardData::BoardData(const QString& boardName,
                      const QString& hostname,
                      const QString& rootPath,
                      const QString& delimiter,
                      const QString& bbsPath,
                      const QString& ext,
                      int boardtype)
: m_boardName(boardName), m_readIdx(false), m_rootPath(rootPath),
    m_delimiter(delimiter), m_bbsPath(bbsPath), m_ext(ext), m_type(boardtype)
{
    // set hostname and create URL of board
    setHostName(hostname);

    // create default key
    QStringList keyHosts(m_hostname);
    createKeys(keyHosts);

    // reset SETTING.TXT
    setSettingLoaded(false);
}

BoardData::~BoardData()
{}


/* public */
void BoardData::setHostName(const QString& hostName)
{
    m_hostname = hostName;

    // m_basePath = (hostname)/(rootPath)/(bbsPath)/
    m_basePath = m_hostname + m_rootPath + m_bbsPath + '/';

    switch (m_type) {

    case Board_MachiBBS: // m_cgiBasePath = (hostname)/(rootPath)/(delimiter)?BBS=(bbsPath)
        m_cgiBasePath = m_hostname + m_rootPath + m_delimiter + "?BBS=" + m_bbsPath.mid(1);
        break;

        // m_cgiBasePath = (hostname)/(rootPath)/(delimiter)/(bbsPath)/
    default:
        m_cgiBasePath = m_hostname + m_rootPath + m_delimiter + m_bbsPath + '/';
        break;
    }
}


/*---------------------------*/
/* information */

/* public */
bool BoardData::readIdx() const
{
    return m_readIdx;
}

/* public */
void BoardData::setReadIdx(bool idx)
{
    m_readIdx = idx;
}

/* public */
const QString& BoardData::boardName() const
{
    return m_boardName;
}

/* public */
const QString& BoardData::hostName() const
{
    return m_hostname;
}

/* public */
const QString& BoardData::rootPath() const
{
    return m_rootPath;
}

/* public */
const QString& BoardData::delimiter() const
{
    return m_delimiter;
}

/* public */
const QString& BoardData::bbsPath() const
{
    return m_bbsPath;
}

/* public */
const QString& BoardData::ext() const
{
    return m_ext;
}

/* public */
int BoardData::type() const
{
    return m_type;
}

/* public */
const QString& BoardData::basePath() const
{
    return m_basePath;
}

/* public */
const QString& BoardData::cgiBasePath() const
{
    return m_cgiBasePath;
}


/*---------------------------*/
/* SETTING.TXT */

/* public */
const QString BoardData::settingUrl() const
{
    return m_basePath + "SETTING.TXT";
}

/* public */
bool BoardData::settingLoaded() const
{
    return m_settingLoaded;
}

/* public */
const QString& BoardData::defaultName() const
{
    return m_defaultName;
}

/* public */
int BoardData::lineNum() const
{
    return m_linenum;
}

/* public */
int BoardData::msgCount() const
{
    return m_msgCount;
}

/* public */
const KUrl& BoardData::titleImgUrl() const
{
    return m_titleImgUrl;
}

/* public */
void BoardData::setSettingLoaded(bool set)
{
    m_settingLoaded = set;
    if (! set) {
            m_defaultName.clear();
            m_linenum = 0;
            m_msgCount = 0;
            m_titleImgUrl.clear();
        }
}

/* public */
void BoardData::setDefaultName(const QString& newName)
{
    m_defaultName = newName;
}

/* public */
void BoardData::setLineNum(int newLine)
{
    m_linenum = newLine;
}

/* public */
void BoardData::setMsgCount(int msgCount)
{
    m_msgCount = msgCount;
}

/* public */
void BoardData::setTitleImgUrl(const KUrl& url)
{
    m_titleImgUrl = url;
}


/*---------------------------*/
/* keys */

/* create keys of DB */ /* public */
void BoardData::createKeys(const QStringList& keyHostList)
{
    // reset keys
    m_keyBasePathList.clear();
    m_keyCgiBasePathList.clear();
    m_keyHostList.clear();

    m_keyHostList = keyHostList;

    // m_basePath = (hostname)/(rootPath)/(bbsPath)/
    for (int i = 0; i < m_keyHostList.count(); ++i) {
        if (m_keyHostList[ i ].length() > 0)
            m_keyBasePathList += m_keyHostList[ i ] + m_rootPath + m_bbsPath + '/';
    }

    switch (m_type) {

    case Board_MachiBBS: // m_cgiBasePath = (hostname)/(rootPath)/(delimiter)?BBS=(bbsPath)
        for (int i = 0; i < m_keyHostList.count(); ++i)
            m_keyCgiBasePathList += m_keyHostList[ i ] + m_rootPath + m_delimiter
                                    + "?BBS=" + m_bbsPath.mid(1);
        break;

        // m_cgiBasePath = (hostname)/(rootPath)/(delimiter)/(bbsPath)/
    default:
        for (int i = 0; i < m_keyHostList.count(); ++i)
            m_keyCgiBasePathList += m_keyHostList[ i ] + m_rootPath + m_delimiter + m_bbsPath + '/';
        break;
    }
}

/* public */
const QStringList& BoardData::keyHostList() const
{
    return m_keyHostList;
}

/* public */
const QStringList& BoardData::keyBasePathList() const
{
    return m_keyBasePathList;
}

/* public */
const QStringList& BoardData::keyCgiBasePathList() const
{
    return m_keyCgiBasePathList;
}
