/***************************************************************************
*   Copyright (C) 2004 by Kita Developers                                 *
*   ikemo@users.sourceforge.jp                                            *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
***************************************************************************/
#ifndef KITABOARDDATA_H
#define KITABOARDDATA_H

#include <QtCore/QList>
#include <QtCore/QStringList>

#include <kurl.h>

namespace Kita
{
    /* type of board */
    enum {
        Board_MachiBBS,
        Board_JBBS,
        Board_2ch,
        Board_FlashCGI,
        Board_Unknown
    };


    /* return value of BoardManager::recordBoard */
    enum {
        Board_recordNew,
        Board_recordRecorded,
        Board_recordMoved,
        Board_recordFailed
    };

    /*---------------------------------*/

    /* Data Base of board */
    class BoardData
    {
        QString m_boardName;
        bool m_readIdx; /* If true, idx file has been read. */

        QString m_hostname;    /* latest host name */
        QString m_rootPath;
        QString m_delimiter;   /* "/test/read.cgi", "/bbs/read.pl", etc. */
        QString m_bbsPath;
        QString m_ext;         /* ".dat", ".cgi", etc.  */
        int m_type;

        QString m_basePath;  /* = (hostname)/(rootPath)/(bbsPath)/ */
        QString m_cgiBasePath;  /* = (hostname)/(rootPath)/(delimiter)/(bbsPath)/ */

        /* SETTING.TXT */
        bool m_settingLoaded;
        QString m_defaultName;
        int m_linenum;
        int m_msgCount;
        KUrl m_titleImgUrl;

        /* keys */
        QStringList m_keyHostList; /* list of host names. They are keys of DB. */
        QStringList m_keyBasePathList;
        QStringList m_keyCgiBasePathList;

    public:
        BoardData(const QString& boardName, const QString& hostname,
                   const QString& rootPath, const QString& delimiter,
                   const QString& bbsPath, const QString& ext, int boardtype);
        ~BoardData();

        void setHostName(const QString& hostname);

        /* information */
        bool readIdx() const;
        void setReadIdx(bool idx);
        const QString& boardName() const;
        const QString& hostName() const;
        const QString& rootPath() const;
        const QString& delimiter() const;
        const QString& bbsPath() const;
        const QString& ext() const;
        int type() const;

        const QString& basePath() const;
        const QString& cgiBasePath() const;

        /* SETTING.TXT */
        const QString settingUrl() const;
        bool settingLoaded() const;
        const QString& defaultName() const;
        int lineNum() const;
        int msgCount() const;
        const KUrl& titleImgUrl() const;
        void setSettingLoaded(bool set);
        void setDefaultName(const QString& newName);
        void setLineNum(int newLine);
        void setMsgCount(int msgCount);
        void setTitleImgUrl(const KUrl& url);

        /* keys */
        void createKeys(const QStringList& keyHostList);
        const QStringList& keyHostList() const;
        const QStringList& keyBasePathList() const;
        const QStringList& keyCgiBasePathList() const;
    };


    /*--------------------------------------*/

    typedef QList<BoardData*> BoardDataList;
}

#endif
