/***************************************************************************
*   Copyright (C) 2010 by Kita Developers                                 *
*   ikemo@users.sourceforge.jp                                            *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
***************************************************************************/
#ifndef KITABOARDDATABASE_H
#define KITABOARDDATABASE_H

#include "boarddata.h"

namespace Kita {
    class Thread;

    class KDE_EXPORT BoardDatabase
    {
        static BoardDataList m_boardDataList;
        static BoardData* m_previousBoardData;
        static QString m_previousBoardUrl;
        static QTextCodec* m_cp932Codec;
        static QTextCodec* m_eucJpCodec;
        KUrl m_url;

    public:
        explicit BoardDatabase(const KUrl& url = KUrl());

        void setUrl(const KUrl& url);
        QString boardUrl();
        QStringList allBoardUrlList();
        QString boardRoot();
        QString boardPath();
        QString ext();
        QString boardId();
        QString subjectUrl();
        QString boardName();
        int type();

        /* ThreadList */
        void getThreadList(bool oldLogs, bool online,
                QList<Thread*>& threadList, QList<Thread*>& oldLogList);

        /* BoardData */
        void clearBoardData();
        int recordBoard(const QString& boardName, QString& oldUrl,
                                int type = Board_Unknown, bool test = false);
        bool isRecorded();
        BoardData* getBoardData();

        /* BBSHISTORY */
        bool loadBBSHistory();
        bool moveBoard(const KUrl& fromUrl, const KUrl& toUrl);

    private:
        /* BoardData */
        BoardData* getBoardData(const KUrl& url);
        int parseBoardUrl(int type, QString& hostname,
                                  QString& rootPath, QString& delimiter,
                                  QString& bbsPath, QString& ext);


        /* ThreadList */
        void getCachedThreadList(const KUrl& url, QList<Thread*>& threadList);
        bool readSubjectTxt(BoardData* bdata, QList<Thread*>& threadList);


        /* SETTING.TXT */
        BoardData* openSettingTxt();
    };
}

#endif
