/***************************************************************************
*   Copyright (C) 2004 by Kita Developers                                 *
*   ikemo@users.sourceforge.jp                                            *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
***************************************************************************/
#ifndef KITADATINFO_H
#define KITADATINFO_H

#include <QtCore/QDateTime>
#include <QtCore/QList>
#include <QtCore/QObject>
#include <QtCore/QVector>

#include <kurl.h>

#include "threadindex.h"

class QStringList;


/* return val of DatInfo::getHTML() */
enum{
    HTML_NOTPARSED,
    HTML_NORMAL,
    HTML_ABONE,
    HTML_BROKEN
};


enum{
    ABONECHK_BY_ID,
    ABONECHK_BY_NAME,
    ABONECHK_BY_WORD
};



/* Data Base of anchor
 
   For example, No.5 res has name = "foo", id "id1234", and anchors >>2-3 and >>4, then,
 
   m_resDatVec[5].name = "foo",
   m_resDatVec[5].id = "id1234",
   m_resDatVec[5].anclist[0].from = 2,
   m_resDatVec[5].anclist[0].to = 3,
   m_resDatVec[5].anclist[1].from = 4,
   m_resDatVec[5].anclist[1].to = 4.   
 
*/
struct ANCNUM
{
    int from;
    int to;
};

typedef QList<ANCNUM> AncList;


/* Data Base of response */
struct RESDAT
{
    int num;         /* number */
    QString linestr; /* raw line data */

    bool parsed;
    bool broken;

    QString name;
    QString nameHTML;
    QString address;
    QString date;
    QDateTime dateTime;
    QString id;
    QString be;
    QString bepointmark;
    QString host;
    QString bodyHTML;

    AncList anclist; /* anchor list. See above. */

    bool checkAbone; /* Is abone checked? */
    bool abone;

    /* Is this res responsed by other one? */
    bool isResponsed;
};
typedef QVector<RESDAT> ResDatVec;


namespace Kita
{

    class Access;
    class OfflawAccess;
    class Thread;

    /*-----------------------*/

    class KDE_EXPORT DatInfo : public QObject
    {
        Q_OBJECT

        /* pointer of Thread class */
        Thread* m_thread;

        /* basic information */
        KUrl m_datUrl;
        ThreadIndex m_threadIndex;
        bool m_broken;
        ResDatVec m_resDatVec;
        bool m_isOpened;

        /* for caching */
        Access* m_access;
        OfflawAccess* m_access2;
        QString m_lastLine;
        bool m_nowLoading;

        /* abone */
        bool m_aboneByID;
        bool m_aboneByName;
        bool m_aboneByBody;
        bool m_aboneChain;


        /* japanese strings */
        QString m_spacestr;
        QString m_framestr1; /* '|' */
        QString m_framestr2; /* '|-' */
        QString m_framestr3; /* 'L' */

        /*-------------------------------*/

    public:

        DatInfo();
        DatInfo(const KUrl& url);
        ~DatInfo();
        void init();
        const KUrl& url() const;

        /* for caching */
        bool updateCache(const QObject* parent);
        int getResponseCode() const;
        int getServerTime() const;
        bool deleteCache();
        bool isLoadingNow() const;
        void stopLoading() const;

        /* string data */
        QString getDat(int num);
        QString getId(int num);
        QString getPlainName(int num);
        QString getPlainTitle(int num);
        QString getPlainBody(int num);

        /* HTML data */
        int getHTML(int num, bool checkAbone, QString& titleHTML,
                QString& bodyHTML);
        QString getHTMLString(int startnum, int endnum,
                bool checkAbone = true);
        QString getHtmlById(const QString& strid, int &count);
        QString getTreeByRes(int rootnum, int& count);
        QString getTreeByResReverse(int rootnum, int& count);

        /* numerical data */
        int getResNum() const;
        int getReadNum() const;
        int getViewPos() const;
        int getNumById(const QString& strid);
        int getDatSize() const;

        /* several information */
        bool isResponsed (int num) const;
        bool isResValid(int num);
        bool isBroken() const;
        bool isResBroken(int num);
        bool checkId(const QString& strid, int num);
        bool checkWord(const QStringList& stlist, int num, bool checkOr);
        bool isOpened() const;
        void setOpened(bool isOpened);

        /* abone check */
        bool checkAbone(int num);
        void resetAbone();

        /*-------------------------*/

    private:

        void initPrivate(bool loadCache = true);
        void resetResDat(RESDAT& resdat) const;
        void increaseResDatVec(int delta);
        void deleteAccessJob();

        /* copy data */
        bool copyOneLineToResDat(const QString& line);

        /* HTML data */
        int getHTMLPrivate(int num, bool checkAbone, QString& titleHTML,
                QString& bodyHTML);
        void getHtmlOfOneRes(int num, bool checkAbone, QString& html);
        QString getTreeByResPrivate(int rootnum, bool reverse, int& count);
        QString getTreeByResCore(int rootnum, bool reverse, int& count,
                const QString& prestr);
        bool checkRes(int num, int target);

        /* for abone */
        void resetAbonePrivate();
        bool checkAbonePrivate(int num);
        bool checkAboneCore(const QString& str, const QStringList& strlist)
            const;

        /* parsing functions */
        bool parseDat(int num);

        void createTitleHtml(RESDAT& resdat, QString& titleHtml);

        DatInfo(const DatInfo&);
        DatInfo& operator=(const DatInfo&);
        /*----------------------------*/

    private slots:

        void slotReceiveData(const QStringList& lineList);
        void slotFinishLoad();

    signals:
        void receiveData();
        void finishLoad();
    };

}

#endif
