/***************************************************************************
*   Copyright (C) 2004 by Kita Developers                                 *
*   ikemo@users.sourceforge.jp                                            *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
***************************************************************************/
#ifndef KITADATMANAGER_H
#define KITADATMANAGER_H

#include <QtCore/QList>

#include <kurl.h>

class QObject;
class QStringList;

namespace Kita
{

    class DatInfo;
    typedef QList<DatInfo*> DatInfoList;

    class KDE_EXPORT DatManager
    {
        static DatInfoList m_datInfoList;

    public:
        explicit DatManager(const KUrl& url);

        bool createDatInfo() const;
        DatInfo* getDatInfoPointer() const;
        static void deleteAllDatInfo();

        /* caching */
        bool updateCache(const QObject* parent) const;
        int getResponseCode() const;
        int getServerTime() const;
        bool deleteCache() const;
        bool isLoadingNow() const;
        void stopLoading() const;

        /* string data */
        QString getDat(int num) const;
        QString getId(int num) const;
        QString getPlainName(int num) const;
        QString getPlainBody(int num) const;
        QString getPlainTitle(int num) const;

        QString threadName() const; /* = subject of thread */
        QString threadId() const;

        /* HTML data */
        QString getHtml(int startnum, int endnum,
                bool checkAbone = true) const;
        QString getHtmlById(const QString& strid, int &count) const;
        QString getTreeByRes(int rootnum, int &count) const;
        QString getTreeByResReverse(int rootnum, int &count) const;

        /* numerical data */
        int getResNum() const;
        int getReadNum() const;
        int getViewPos() const;
        void setViewPos(int num) const;
        int getDatSize() const;
        int getNumById(const QString& strid) const;


        /* another information */
        bool isThreadRecorded() const;
        bool is2chThread() const;
        bool isResValid(int num) const;
        bool isBroken() const;
        bool isResBroken(int num) const;
        bool checkId(const QString& strid, int num) const;
        bool checkWord(const QStringList& stlist, int num, bool checkOr) const;
        bool isMarked(int num) const;
        void setMark(int num, bool mark) const;


        /* abone check */
        bool checkAbone(int num) const;
        void resetAbone() const;

        /* check if the thread is shown on the main thread tab. */
        bool isMainThreadOpened() const;
        void setMainThreadOpened(bool isOpened) const;

        /* obsolete. Don't use them. */

        QString threadUrl() const;

    private:
        DatInfo* getDatInfo(bool checkCached = true) const;
        DatInfo* searchDatInfo() const;
        DatInfo* recordDatInfo(bool checkCached) const;

        KUrl m_url;
        KUrl m_datUrl;
        DatInfo* m_datInfo;
        DatInfo* m_searchDatInfo;

        DatManager(const DatManager&);
        DatManager& operator=(const DatManager&);
    };

}

#endif
