/***************************************************************************
*   Copyright (C) 2004 by Kita Developers                                 *
*   ikemo@users.sourceforge.jp                                            *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
***************************************************************************/
#include "favoriteboards.h"

#include <QtXml/QDomDocument>

#include "boarddatabase.h"

using namespace Kita;

FavoriteBoards* FavoriteBoards::instance = 0;

FavoriteBoards::FavoriteBoards()
{}

FavoriteBoards::~FavoriteBoards()
{}

FavoriteBoards* FavoriteBoards::getInstance()
{
    if (! instance) {
        instance = new FavoriteBoards();
    }
    return instance;
}

void FavoriteBoards::append(KUrl& url)
{
    if (! getInstance() ->m_list.contains(url)) {
        getInstance() ->m_list.append(url);
        getInstance() ->notifyChange();
    }
}

void FavoriteBoards::remove(KUrl& url)
{
    if (getInstance() ->m_list.contains(url)) {
        getInstance() ->m_list.removeAll(url);
        getInstance() ->notifyChange();
    }
}

const QList<KUrl>& FavoriteBoards::boards()
{
    return getInstance() ->m_list;
}

bool FavoriteBoards::readFromXML(QString& xml)
{
    FavoriteBoards * instance = FavoriteBoards::getInstance();
    instance->m_list.clear();

    QDomDocument document;
    if (! document.setContent(xml, true)) {
        return false;
    }
    QDomElement root = document.documentElement();

    QDomNode node = root.firstChild();
    while (! node.isNull()) {
        if (node.isElement() &&
                node.nodeName() == QString("board") &&
                node.namespaceURI() == QString("http://kita.sourceforge.jp/ns/board")) {
            processChildNode(node);
        }
        node = node.nextSibling();
    }
    return true;
}

void FavoriteBoards::processChildNode(QDomNode& node)
{
    QDomNode urlNode = node.namedItem("url");
    if (! urlNode.isElement()) return ;

    QString urlText = urlNode.toElement().text();

    KUrl url = KUrl(urlText);
    if (url.isValid()) {
        //FavoriteBoards::append(url);
        if (!getInstance() ->m_list.contains(url))
            getInstance() ->m_list.append(url);
    }
}

QString FavoriteBoards::toXML()
{
    QDomDocument document;

    QDomProcessingInstruction pi = document.createProcessingInstruction("xml", "version=\"1.0\"");
    document.appendChild(pi);

    QDomElement root = document.createElementNS("http://kita.sourceforge.jp/ns/boardlist", "boardlist");
    document.appendChild(root);

    QList<KUrl> boards = FavoriteBoards::boards();
    QList<KUrl>::iterator it;
    for (it = boards.begin(); it != boards.end(); ++it) {
        QDomElement board = document.createElementNS("http://kita.sourceforge.jp/ns/board", "board");
        root.appendChild(board);

        QString boardUrl = (*it).url();
        QDomElement urlElement = document.createElement("url");
        board.appendChild(urlElement);
        urlElement.appendChild(document.createTextNode(boardUrl));

        BoardDatabase db(boardUrl);
        QString boardName = db.boardName();
        QDomElement nameElement = document.createElement("name");
        board.appendChild(nameElement);
        nameElement.appendChild(document.createTextNode(boardName));
    }
    return document.toString(0);
}

void FavoriteBoards::replace(const QString& fromUrl, const QString& toUrl)
{
    if (FavoriteBoards::getInstance() == 0) return ;
    QList<KUrl>& boardList = FavoriteBoards::getInstance() ->m_list;
    for (QList<KUrl>::iterator it = boardList.begin(); it != boardList.end(); ++it) {
        QString url = (*it).url();
        if (url.startsWith(fromUrl)) {
            url = url.replace(0, fromUrl.length(), toUrl);
            *it = url;
        }
    }
    FavoriteBoards::getInstance() ->notifyChange();
}

void FavoriteBoards::notifyChange()
{
    emit changed();
}
