/***************************************************************************
*   Copyright (C) 2004 by Kita Developers                                 *
*   ikemo@users.sourceforge.jp                                            *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
***************************************************************************/

#include "favoritethreads.h"

#include <QtCore/QList>
#include <QtXml/QDomDocument>

#include "boarddatabase.h"
#include "datmanager.h"
#include "kita_misc.h"
#include "thread.h"
#include "threadindex.h"

using namespace Kita;

// FavoriteThreadItem
FavoriteThreadItem::FavoriteThreadItem()
{}

FavoriteThreadItem::~ FavoriteThreadItem()
{}

bool FavoriteThreadItem::operator==(const FavoriteThreadItem& item) const
{
    return (m_datUrl == item.m_datUrl);
}

// FavoriteThreads
FavoriteThreads* FavoriteThreads::instance = 0;

FavoriteThreads::FavoriteThreads()
{}

FavoriteThreads::~FavoriteThreads()
{}

FavoriteThreads* FavoriteThreads::getInstance()
{
    if (! instance) {
        instance = new FavoriteThreads();
    }
    return instance;
}

void FavoriteThreads::insert(const QString& datUrl)
{
    if (! m_threadList.contains(datUrl)) {
        m_threadList.append(datUrl);
    }
}

void FavoriteThreads::remove(const QString& datUrl)
{
    m_threadList.removeAll(datUrl);
}

bool FavoriteThreads::contains(const QString& datUrl)
{
    if (m_threadList.contains(datUrl)) {
        return true;
    } else {
        return false;
    }
}

const QList<FavoriteThreadItem> FavoriteThreads::threadList() const
{
    return m_threadList;
}

bool FavoriteThreads::readFromXML(const QString& xml)
{
    FavoriteThreads * instance = FavoriteThreads::getInstance();
    instance->m_threadList.clear();

    QDomDocument document;
    if (! document.setContent(xml, true)) {
        return false;
    }

    QDomElement root = document.documentElement();

    QDomNode node = root.firstChild();
    while (!node.isNull()) {
        if (node.isElement() &&
                node.nodeName() == QString("thread") &&
                node.namespaceURI() == QString("http://kita.sourceforge.jp/ns/thread")) {
            processThreadNode(node);
        }
        node = node.nextSibling();
    }
    return true;
}

void FavoriteThreads::processThreadNode(QDomNode& node)
{
    QDomNode datUrlNode = node.namedItem("daturl");
    QDomNode nameNode = node.namedItem("name");

    if (!datUrlNode.isElement() || !nameNode.isElement()) return ;

    QString url = datUrlNode.toElement().text();
    QString name = nameNode.toElement().text();

    KUrl datUrl = Kita::getDatUrl(url);
    Thread* thread = Thread::getByUrl(datUrl);
    thread->setThreadName(name);
    ThreadIndex threadIndex(datUrl);
    threadIndex.loadIndex(thread);

    FavoriteThreads::getInstance() ->insert(datUrl.prettyUrl());
}

const QString FavoriteThreads::toXML() const
{
    QDomDocument document;

    //    QDomProcessingInstruction pi = document.createProcessingInstruction("xml", "version=\"1.0\"");
    //    document.appendChild(pi);

    QDomElement root = document.createElementNS("http://kita.sourceforge.jp/ns/favorites", "favorites");
    document.appendChild(root);

    QList<FavoriteThreadItem>::const_iterator it;
    for (it = threadList().begin(); it != threadList().end(); ++it) {
        QString datUrl = (*it).m_datUrl;
        QDomElement threadElement = document.createElementNS("http://kita.sourceforge.jp/ns/thread", "thread");
        root.appendChild(threadElement);

        QDomElement datUrlElement = document.createElement("daturl");
        threadElement.appendChild(datUrlElement);
        datUrlElement.appendChild(document.createTextNode(datUrl));

        DatManager datManager(datUrl);
        QString threadName = datManager.threadName();
        QDomElement nameElement = document.createElement("name");
        threadElement.appendChild(nameElement);
        nameElement.appendChild(document.createTextNode(threadName));

        QString resNum = QString::number(datManager.getResNum());
        QDomElement resNumElement = document.createElement("resnum");
        threadElement.appendChild(resNumElement);
        resNumElement.appendChild(document.createTextNode(resNum));

        // board
        QDomElement board = document.createElementNS("http://kita.sourceforge.jp/ns/board", "board");
        threadElement.appendChild(board);

        BoardDatabase db(datUrl);
        QString boardUrl = db.boardUrl();
        QDomElement boardUrlElement = document.createElement("url");
        board.appendChild(boardUrlElement);
        boardUrlElement.appendChild(document.createTextNode(boardUrl));

        BoardDatabase db2(boardUrl);
        QString boardName = db2.boardName();
        QDomElement boardNameElement = document.createElement("name");
        board.appendChild(boardNameElement);
        boardNameElement.appendChild(document.createTextNode(boardName));
    }
    return document.toString(0);
}

void FavoriteThreads::replace(const QString& fromUrl, const QString& toUrl)
{
    if (FavoriteThreads::getInstance() == 0) return ;
    QList<FavoriteThreadItem>& threadList = FavoriteThreads::getInstance() ->m_threadList;
    QList<FavoriteThreadItem>::iterator it;
    for (it = threadList.begin(); it != threadList.end(); ++it) {
        QString url = (*it).m_datUrl;
        if (url.indexOf(fromUrl) == 0) {
            url = url.replace(0, fromUrl.length(), toUrl);
            threadList.erase(it);
            threadList.prepend(url);
            it = threadList.begin();
        }
    }
}

QString FavoriteThreads::getDatUrl(int i)
{
    if (getInstance() ->m_threadList.count() > i) {
        return getInstance() ->m_threadList[ i ].m_datUrl;
    } else {
        return QString();
    }
}
