/***************************************************************************
*   Copyright (C) 2004 by Kita Developers                                 *
*   ikemo@users.sourceforge.jp                                            *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
***************************************************************************/

#include "kita_misc.h"

#include <QtCore/QRegExp>
#include <QtGui/QFont>

#include <kurl.h>

#include "boarddatabase.h"
#include "datmanager.h"
#include "kita-utf16.h"

using namespace Kita;

/* fro convertURL */
static int m_prevConvMode;
static QString m_prevConvUrl;
static QString m_prevConvNewUrl;
static QString m_prevConvRefstr;

/* for ParseMachiBBSOneLine */
static QString m_machiSubject;
static QString m_machiLine;

/*------------------------------------------------------------*/
/*------------------------------------------------------------*/

/* conversion of URL */

KUrl Kita::getDatUrl(const KUrl& url , QString& refstr)
{
    return convertUrl(URLMODE_DAT, url, refstr);
}

KUrl Kita::getDatUrl(const KUrl& url)
{
    QString refstr;
    return convertUrl(URLMODE_DAT, url, refstr);
}

QString getThreadUrl(const KUrl& url, QString& refstr)
{
    return convertUrl(URLMODE_THREAD, url, refstr);
}

QString Kita::getThreadUrl(const KUrl& url)
{
    QString refstr;
    return convertUrl(URLMODE_THREAD, url, refstr);
}

QString Kita::getWriteUrl(const KUrl& m_datUrl)
{
    BoardDatabase db(m_datUrl);
    int m_bbstype = db.type();
    QString m_bbscgi;

    // set path of bbs.cgi
    switch (m_bbstype) {

    case Board_JBBS: {
            QString cgipath = db.boardRoot()
                + "/bbs/write.cgi/" + db.boardId() + '/';

            cgipath += DatManager(m_datUrl).threadId() + '/';

            m_bbscgi = cgipath;
        }

        break;

    case Board_MachiBBS: {
            QString cgipath = db.boardRoot()
                              + "/bbs/write.cgi";
            m_bbscgi = cgipath;
        }

        break;

    default:

        m_bbscgi = db.boardRoot() + "/test/bbs.cgi";
    }

    return m_bbscgi;
}

/* convert thread URL, and get reference.
   If mode = URLMODE_DAT, output is URL of dat file.
   If mode = URLMODE_THREAD, output is URL of read.cgi .
 
   If url is NOT recorded, return QString(). 
   
(ex.1)
 
mode = 0
url  = http://pc5.2ch.net/linux/dat/1069738960.dat#20-30
->
return : http://pc5.2ch.net/linux/dat/1069738960.dat
retstr : 20-30
 
(ex.2)
 
mode = 1
url  = http://pc5.2ch.net/linux/dat/1069738960.dat#20-30
->
return : http://pc5.2ch.net/test/read.cgi/linux/1069738960
retstr : 20-30
 
(ex.3)
 
mode = 0
url  = http://pc5.2ch.net/test/read.cgi/linux/1069738960/-100
->
return : http://pc5.2ch.net/linux/dat/1069738960.dat
refstr : 1-100
                                                            */

QString Kita::convertUrl(

    /* input */
    int mode,   /* if 0, output is dat URL. If 1, output is thread URL */
    const KUrl& url ,

    /* output */
    QString& refstr)
{
    refstr.clear();

    if (url.isEmpty()) return QString();

    // cache
    if (m_prevConvMode == mode && m_prevConvUrl == url.prettyUrl()) {

        refstr = m_prevConvRefstr;
        return m_prevConvNewUrl;
    }

    // Is board recorded?
    BoardDatabase db(url);
    BoardData* bdata = db.getBoardData();
    if (bdata == 0) return QString();

    QString urlstr = url.prettyUrl();

    // get thread & reference
    QString thread;
    QString refBase;

    if (urlstr.contains("/dat/")) {

        // url = (hostname)/(rootPath)/(bbsPath)/dat/(thread_ID).(ext)#(refBase)
        thread = url.fileName().remove(bdata->ext());
        refBase = url.ref();
    } else if (urlstr.contains(bdata->delimiter())) {

        QString tmpstr;
        switch (bdata->type()) {

            // machi BBS
            // ex.) If url = http://kanto.machi.to/bbs/read.pl?BBS=kana&KEY=1096716679 ,
            //   then, thread = 1096716679
        case Board_MachiBBS:
            thread = url.queryItem("KEY");
            refBase.clear();
            break;

            // url = (hostname)/(rootPath)/(delimiter)/(bbsPath)/(thread_ID)/(refBase)
        default:
            tmpstr = urlstr.section(bdata->delimiter() + bdata->bbsPath(), 1, 1);
            thread = tmpstr.section('/', 1, 1);
            refBase = tmpstr.section('/', 2, 2);
            break;
        }
    }

    if (thread.isEmpty()) return QString();

    if (!refBase.isEmpty()) {

        if (refBase.at(0) == '-') refstr = '1' + refBase;
        else refstr = refBase;
    }

    // create new URL
    QString newUrl;
    if (mode == URLMODE_DAT) newUrl = bdata->basePath() + "dat/" + thread + bdata->ext();
    else {
        newUrl = bdata->cgiBasePath();

        switch (bdata->type()) {

        case Board_MachiBBS:
            newUrl += "&KEY=" + thread;
            break;

        default:
            newUrl += thread;
            break;
        }
    }

    // cache
    m_prevConvMode = mode;
    m_prevConvUrl = url.prettyUrl();
    m_prevConvNewUrl = newUrl;
    m_prevConvRefstr = refstr;

    return newUrl;
}



/**
 * http://pc5.2ch.net/linux/dat/1089905503.dat
 * -> http://pc5.2ch.net/test/offlaw.cgi?bbs=linux&key=1089905503
 */
QString Kita::datToOfflaw(const KUrl& datUrl)
{
    // TODO: not tested.
    KUrl url(datUrl);
    QString root = url.host();

    QStringList list = url.fileName().split('.');
    if (list.size() != 2) {
        return QString();
    }
    QString datName = list[ 0 ];

    url.cd("..");
    if (url.fileName() != "dat") {
        return QString();
    }

    url.cd("..");
    QString board = url.fileName();

    return QString("http://%1/test/offlaw.cgi?raw=0.0&bbs=%2&key=%3").arg(root).arg(board).arg(datName);
}


/*------------------------------------------------------------*/
/*------------------------------------------------------------*/

/* utilities */

uint Kita::datToSince(const KUrl& datUrl)
{
    return KUrl(datUrl).fileName().section('.', 0, 0).toInt();
}



/* convert strings to positive number.  */
/* if cdat is not number, return -1.    */

/*  For example, if cdat = "1234", then
    ret = 1234. If cdat = "abcd", then
    ret = -1.                           */
int Kita::stringToPositiveNum(const QChar *cdat, const unsigned int length)
{
    int ret = 0;

    for (unsigned int i = 0 ; i < length ; i++) {

        unsigned short c = cdat[ i ].unicode();

        if ((c < UTF16_0 || c > UTF16_9) && (c < '0' || c > '9')) return -1;

        ret *= 10;
        if (c >= UTF16_0) ret += c - UTF16_0;
        else ret += c - '0';
    }

    return ret;
}



/*------------------------------------------------------------*/
/*------------------------------------------------------------*/

/* internal parsing functions  */

QStringList Kita::parseSearchQuery(const QString& input)
{
    QStringList tmp = input.split(' ');
    QStringList ret_list;
    QRegExp truncSpace("\\s*$");
    QStringList::iterator it = tmp.begin();
    for (; it != tmp.end(); ++it)
        ret_list += (*it).remove(truncSpace);
    return ret_list;
}


/* for Machi BBS */


/* init parser. Don't forget to call this before parsing. */
void Kita::InitParseMachiBBS()
{
    m_machiSubject.clear();
    m_machiLine.clear();
}

QString Kita::ParseMachiBBSOneLine(const QString& inputLine, int& nextNum)
{
    QString ret;
    m_machiLine += inputLine;

    int num = 0;
    QString name;
    QString mail;
    QString date;
    QString time;
    QString id;
    QString host;
    QString message;

    // Subject
    QRegExp title_regexp("<title>(.*)</title>");

    // pattern 1 (tokyo,kanagawa,...)
    QRegExp regexp ("<dt>(\\d*) .*<font color=\"#......\"><b> (.*) </b></font> .* (..../../..).* (..:..:..) ID:([^<]*)<br><dd>(.*)");
    QRegExp regexp2("<dt>(\\d*) .*<a href=\"mailto:(.*)\"><b> (.*) </B></a> .* (..../../..).* (..:..:..) ID:([^<]*)<br><dd>(.*)");

    // pattern 2 (hokkaido,...)
    QRegExp regexp3("<dt>(\\d*) .*<font color=\"#......\"><b> (.*) </b></font> .* (..../../..).* (..:..:..) ID:([^<]*) <font size=.>\\[ ([^ ]*) \\]</font><br><dd>(.*)");
    QRegExp regexp4("<dt>(\\d*) .*<a href=\"mailto:(.*)\"><b> (.*) </B></a> .* (..../../..).* (..:..:..) ID:([^<]*) <font size=.>\\[ ([^ ]*) \\]</font><br><dd>(.*)");

    // abone
    QRegExp regexp5("<dt>(\\d*) .*<br><dd>.*");


    if (regexp.indexIn(m_machiLine) != -1) {

        num = regexp.cap(1).toInt();
        name = regexp.cap(2);
        date = regexp.cap(3);
        time = regexp.cap(4);
        id = regexp.cap(5);
        message = regexp.cap(6);

    } else if (regexp2.indexIn(m_machiLine) != -1) {

        num = regexp2.cap(1).toInt();
        mail = regexp2.cap(2);
        name = regexp2.cap(3);
        date = regexp2.cap(4);
        time = regexp2.cap(5);
        id = regexp2.cap(6);
        message = regexp2.cap(7);

    } else if (regexp3.indexIn(m_machiLine) != -1) {

        num = regexp3.cap(1).toInt();
        name = regexp3.cap(2);
        date = regexp3.cap(3);
        time = regexp3.cap(4);
        id = regexp3.cap(5);
        host = regexp3.cap(6);
        message = regexp3.cap(7);

    } else if (regexp4.indexIn(m_machiLine) != -1) {

        num = regexp4.cap(1).toInt();
        mail = regexp4.cap(2);
        name = regexp4.cap(3);
        date = regexp4.cap(4);
        time = regexp4.cap(5);
        id = regexp4.cap(6);
        host = regexp4.cap(7);
        message = regexp4.cap(8);

    } else if (regexp5.indexIn(m_machiLine) != -1) { // abone

        num = regexp5.cap(1).toInt();
        m_machiLine.clear();
        if (num == nextNum) return "abone<><><>abone<>";
        else return QString();

    } else if (title_regexp.indexIn(m_machiLine) != -1) { // get title

        m_machiSubject = title_regexp.cap(1);
        m_machiLine.clear();
        return QString();
    }
    else return QString();

    if (num >= nextNum) {

        if (num != 1) m_machiSubject.clear();
        ret += name + "<><>" + date + ' ' + time + " ID:" + id;
        if (!host.isEmpty()) ret += " HOST:" + host;
        ret += "<>" + message + "<>" + m_machiSubject;
        nextNum = num;
    }

    m_machiLine.clear();
    return ret;
}


/*---------------------------------*/

/* for JBBS */

QString Kita::ParseJBBSOneLine(const QString& line, int& nextNum)
{
    QString ret;
    QStringList list = line.split("<>");
    if (list.size() != 7) return QString();

    int num = list[ 0 ].toInt();
    QString name = list[ 1 ];
    QString mail = list[ 2 ];
    QString date = list[ 3 ];
    QString body = list[ 4 ];
    QString subject = list[ 5 ];
    QString id = list[ 6 ];

    if (num < nextNum) return QString();

    // remove tag
    QRegExp rex("<[^<]*>");
    name.remove(rex);

    // remove week
    rex = QRegExp("\\(.*\\)");
    date.remove(rex);

    ret += name + "<>" + mail + "<>" + date + " ID:" + id + "<>" + body + "<>" + subject;
    nextNum = num;

    return ret;
}


/*---------------------------------*/

/* for Flash CGI/Mini Thread  */

QString Kita::ParseFlashCGIOneLine(const QString& line)
{
    QString ret;
    QStringList list = line.split("<>");
    if (list.size() != 13) return QString();

    QString name = list[ 0 ];
    QString mail = list[ 1 ];
    QString date = list[ 2 ];
    QString body = list[ 3 ];
    QString subject = list[ 4 ];
    QString id = list[ 6 ];
    QString host = list[ 7 ];

    // remove tag
    QRegExp rex("<[^<]*>");
    name.remove(rex);

    ret += name + "<>" + mail + "<>" + date + " ID:" + id;
    if (!host.isEmpty()) ret += " HOST:" + host;
    ret += "<>" + body + "<>" + subject;

    return ret;
}


/*-------------------------------------------------*/
/*-------------------------------------------------*/

/* parsing functions */

/*-------------------------------------------------*/
/*-------------------------------------------------*/

QString Kita::getCategory(const QString& line)
{
    QRegExp regexp("<BR><BR><B>(.*)</B><BR>");
    if (regexp.indexIn(line) != -1) {
        return regexp.cap(1);
    } else {
        return QString();
    }
}

bool Kita::isBoardUrl(const QString& url)
{
    QRegExp url_2ch("http://.*\\.2ch\\.net/.*");
    QRegExp url_bbspink("http://.*\\.bbspink\\.com/.*");
    QRegExp url_www_2ch("http://www\\.2ch\\.net/.*");
    QRegExp url_machibbs("http://.*\\.machi\\.to/.*");

    if (url.isEmpty()) return false;

    if (url_2ch.indexIn(url) == -1 && url_bbspink.indexIn(url) == -1
            && url_machibbs.indexIn(url) == -1) return false;
    if (url_www_2ch.indexIn(url) != -1) return false;

    return true;
}

QString Kita::fontToString(const QFont& font)
{
    return font.family() + ' ' + QString::number(font.pointSize());
}
