/***************************************************************************
*   Copyright (C) 2003 by Hideki Ikemoto                                  *
*   ikemo@users.sourceforge.jp                                            *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
***************************************************************************/

#include "mainwindow.h"

#include <cstdlib> // setenv()

#include <kaboutdata.h>
#include <kcmdlineargs.h>
#include <kuniqueapplication.h>

#include "libkita/config.h"
#include "libkita/globalconfig.h"

static const char *description =
    I18N_NOOP("Kita - 2ch client for KDE");

using namespace Kita;

int main(int argc, char **argv)
{
    // for code page 932 with NEC special characters
    setenv("UNICODEMAP_JP", "cp932,nec-vdc", 1);
    KAboutData about("kita", "kita", ki18n("Kita"), VERSION, ki18n(description),
        KAboutData::License_GPL, ki18n("(C) 2003-2010 Kita Developers"),
        KLocalizedString(), QByteArray(), "ikemo@users.sourceforge.jp");
    about.addAuthor(ki18n("Hideki Ikemoto"), ki18n("maintainer, initial code"),
        "ikemo@users.sourceforge.jp");
    about.addAuthor(ki18n("konqueror plugin no hito"),
        ki18n("konqueror plugin, KDE part"), "ogirin@users.sourceforge.jp");
    about.addAuthor(ki18n("421"), ki18n("kitanavi, threadview's improvement"));
    about.addAuthor(ki18n("Toshihiko Okada"), ki18n("improvements"),
        "tossi@users.sourceforge.jp");
    about.addAuthor(ki18n("75"), ki18n("stylesheet support"));
    KCmdLineArgs::init(argc, argv, &about);
    KCmdLineOptions options;
    options.add("+[URL]", ki18n("Document to open."));
    options.add("boardlist <boardlist>", ki18n("board list's URL."),
        "http://menu.2ch.net/bbsmenu.html");
    KCmdLineArgs::addCmdLineOptions(options);
    KUniqueApplication app;

    // register ourselves as a dcop client
//    app.dcopClient() ->registerAs(app.name(), FALSE);

    KCmdLineArgs* args = KCmdLineArgs::parsedArgs();
    GlobalConfig::setBoardListUrl(QString(args->getOption("boardlist")));

    // see if we are starting with session management
    if (app.isSessionRestored()) {
        kRestoreMainWindows<MainWindow>();
    } else {
            // no session.. just start up normally
            KCmdLineArgs * args = KCmdLineArgs::parsedArgs();
            if (args->count() == 0) {
                MainWindow * widget = new MainWindow;
                widget->show();
            } else {
                int i = 0;
                for (; i < args->count(); i++) {
                    MainWindow *widget = new MainWindow;
                    widget->show();
                    widget->load(args->url(i));
                }
            }
            args->clear();
    }

    return app.exec();
}
