/***************************************************************************
*   Copyright (C) 2004 by Kita Developers                                 *
*   ikemo@users.sourceforge.jp                                            *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
***************************************************************************/
#ifndef KITAMAINWINDOW_H
#define KITAMAINWINDOW_H

#include <kxmlguiwindow.h>

class KLineEdit;
class KToggleAction;
class KUrl;

namespace Kita {
    class BBSTabWidget;
    class BoardTabWidget;
    class ThreadTabWidget;
    class WriteTabWidget;
    class Preferences;

/**
 * This class serves as the main window for Kita.  It handles the
 * menus, toolbars, and status bars.
 *
 * @short Main window class
 * @author $AUTHOR <$EMAIL>
 * @version $APP_VERSION
 */

    class MainWindow : public KXmlGuiWindow
    {
        Q_OBJECT
    public:
        MainWindow();
        virtual ~MainWindow();

        void load(const KUrl& url);
        void setMainStatus(const QString& statusStr);
        void setUrl(const KUrl& url);
        void bookmark(const QString& datUrl, bool on);

    private slots:
        void login();
        void newToolbarConfig();
        void optionsShowToolbar();
        void optionsShowStatusbar();
        void optionsConfigureKeys();
        void optionsConfigureToolbars();
        void optionsPreferences();
        void settingsMenuAboutToShow();
        void setFont(const QFont& font);

        void slotEditCopy();
        void slotUrlLine();

    private:
        KLineEdit* m_urlLine;

        KToggleAction *m_toolbarAction;
        KToggleAction *m_statusbarAction;

        ThreadTabWidget* m_threadTab;
        BBSTabWidget* m_bbsTab;
        BoardTabWidget* m_boardTab;
        WriteTabWidget* m_writeTab;

        Preferences *m_prefDialog;
        void setupActions();
        void setupView();
        void loadCache();
        void saveCache();
        void loadFavorites();
        void saveFavorites();
        void loadFavoriteBoards();
        void saveFavoriteBoards();
        void loadCompletion();
        void loadAsciiArt();
        void loadAboneIDList();
        void loadAboneNameList();
        void loadAboneWordList();

        MainWindow(const MainWindow&);
        MainWindow& operator=(const MainWindow&);
    };
}

#endif // MAINWINDOW_H
