/***************************************************************************
*   Copyright (C) 2003 by Hideki Ikemoto                                  *
*   ikemo@users.sourceforge.jp                                            *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
***************************************************************************/

#include "faceprefpage.h"

#include <kfontdialog.h>

#include "libkita/colorconfig.h"
#include "libkita/globalconfig.h"
#include "libkita/kita_misc.h"

using namespace Kita;

FacePrefPage::FacePrefPage(QWidget* parent) : AbstractPrefPage(parent)
{
    setupUi(this);
    load();
    // font
    connect(listFontButton, SIGNAL(clicked()), SLOT(slotFontButtonClicked()));

    connect(threadFontButton, SIGNAL(clicked()),
             SLOT(slotThreadFontButtonClicked()));

    connect(popupFontButton, SIGNAL(clicked()),
             SLOT(slotPopupFontButtonClicked()));

    // color
    connect(threadColorButton, SIGNAL(changed(const QColor&)),
            SIGNAL(changed()));
    connect(threadBackgroundColorButton, SIGNAL(changed(const QColor&)),
            SIGNAL(changed()));
    connect(popupColorButton, SIGNAL(changed(const QColor&)),
            SIGNAL(changed()));
    connect(popupBackgroundColorButton, SIGNAL(changed(const QColor&)),
            SIGNAL(changed()));
}

void FacePrefPage::apply()
{
    // font
    QFont font = listFontButton->font();
    GlobalConfig::setFont(font);
    emit fontChanged(font);

    QFont threadFont = threadFontButton->font();
    GlobalConfig::setThreadFont(threadFont);

    QFont popupFont = popupFontButton->font();
    GlobalConfig::setPopupFont(popupFont);

    // color
    ColorConfig::setThread(threadColorButton->color());
    ColorConfig::setThreadBackground(threadBackgroundColorButton->color());
    ColorConfig::setPopup(popupColorButton->color());
    ColorConfig::setPopupBackground(popupBackgroundColorButton->color());
}

void FacePrefPage::load()
{
    // font
    QFont font = GlobalConfig::font();
    listFontButton->setText(fontToString(font));
    listFontButton->setFont(font);

    QFont threadFont = GlobalConfig::threadFont();
    threadFontButton->setText(fontToString(threadFont));
    threadFontButton->setFont(threadFont);

    QFont popupFont = GlobalConfig::popupFont();
    popupFontButton->setText(fontToString(popupFont));
    popupFontButton->setFont(popupFont);

    // color
    threadColorButton->setColor(ColorConfig::thread());
    threadBackgroundColorButton->setColor(ColorConfig::threadBackground());
    popupColorButton->setColor(ColorConfig::popup());
    popupBackgroundColorButton->setColor(ColorConfig::popupBackground());

}

void FacePrefPage::reset()
{
    ColorConfig::self()->useDefaults(true);
    GlobalConfig::self()->useDefaults(true);
    load();
    GlobalConfig::self()->useDefaults(false);
    ColorConfig::self()->useDefaults(false);
}

void FacePrefPage::slotThreadFontButtonClicked()
{
    QFont threadFont = threadFontButton->font();

    if (KFontDialog::getFont(threadFont, false, this) == QDialog::Accepted) {
        threadFontButton->setText(fontToString(threadFont));
        threadFontButton->setFont(threadFont);
        emit changed();
    }
}

void FacePrefPage::slotFontButtonClicked()
{
    QFont font = listFontButton->font();

    if (KFontDialog::getFont(font, false, this) == QDialog::Accepted) {
        listFontButton->setText(fontToString(font));
        listFontButton->setFont(font);
        emit changed();
    }
}

void FacePrefPage::slotPopupFontButtonClicked()
{
    QFont font = popupFontButton->font();

    if (KFontDialog::getFont(font, false, this) == QDialog::Accepted) {
        popupFontButton->setText(fontToString(font));
        popupFontButton->setFont(font);
        emit changed();
    }
}
