/***************************************************************************
*   Copyright (C) 2003 by Hideki Ikemoto                                  *
*   ikemo@users.sourceforge.jp                                            *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
***************************************************************************/

#include "preferences.h"

#include "aboneprefpage.h"
#include "asciiartprefpage.h"
#include "faceprefpage.h"
#include "loginprefpage.h"
#include "uiprefpage.h"
#include "writeprefpage.h"
#include "libkita/globalconfig.h"

using namespace Kita;

Preferences::Preferences(QWidget* parent) : KPageDialog(parent)
{
    setWindowTitle("Kita Preferences");

    setButtons(Ok | Apply | Cancel | Default);
    enableButtonApply(false);
    setDefaultButton(Ok);

    // this is the base class for your preferences dialog.  it is now
    // a Treelist dialog.. but there are a number of other
    // possibilities (including Tab, Swallow, and just Plain)
    FacePrefPage* facePage = new FacePrefPage(0);
    KPageWidgetItem* faceItem
        = addPage(facePage, i18n("Face"));
    faceItem->setIcon(KIcon("view-list-details"));
    connect(facePage, SIGNAL(fontChanged(const QFont&)),
             SIGNAL(fontChanged(const QFont&)));
    connect(facePage, SIGNAL(changed()), SLOT(slotChanged()));
    m_pageList.append(facePage);

    AsciiArtPrefPage* asciiArtPage = new AsciiArtPrefPage(0);
    KPageWidgetItem* asciiArtItem
        = addPage(asciiArtPage, i18n("AsciiArt"));
    asciiArtItem->setIcon(KIcon("kita"));
    connect(asciiArtPage, SIGNAL(changed()), SLOT(slotChanged()));
    m_pageList.append(asciiArtPage);

    UIPrefPage* uiPage = new UIPrefPage(0);
    KPageWidgetItem* configureItem
        = addPage(uiPage, i18n("User Interface"));
    configureItem->setIcon(KIcon("configure"));
    connect(uiPage, SIGNAL(changed()), SLOT(slotChanged()));
    m_pageList.append(uiPage);

    AbonePrefPage* abonePage = new AbonePrefPage(0);
    KPageWidgetItem* aboneItem = addPage(abonePage, i18n("Abone"));
    aboneItem->setIcon(KIcon("kita"));
    connect(abonePage, SIGNAL(changed()), SLOT(slotChanged()));
    m_pageList.append(abonePage);

    LoginPrefPage* loginPage = new LoginPrefPage(0);
    KPageWidgetItem* loginItem
        = addPage(loginPage, i18n("Login"));
    loginItem->setIcon(KIcon("network-connect"));
    connect(loginPage, SIGNAL(changed()), SLOT(slotChanged()));
    m_pageList.append(loginPage);

    WritePrefPage* writePage = new WritePrefPage(0);
    KPageWidgetItem* writeItem
        = addPage(writePage, i18n("Write"));
    writeItem->setIcon(KIcon("document-edit"));
    connect(writePage, SIGNAL(changed()), SLOT(slotChanged()));
    m_pageList.append(writePage);
}

void Preferences::load()
{
    AbstractPrefPage* page;
    foreach (page, m_pageList) {
        page->load();
    }
    enableButtonApply(false);
}

void Preferences::apply()
{
    AbstractPrefPage* page;
    foreach (page, m_pageList) {
        page->apply();
    }
    enableButtonApply(false);
}

void Preferences::reset()
{
    AbstractPrefPage* page;
    foreach (page, m_pageList) {
        page->reset();
    }
}

void Preferences::slotButtonClicked(int button)
{
    if (button == Ok || button == Apply) {
        apply();
    } else if (button == Default) {
        reset();
    }
    KPageDialog::slotButtonClicked(button);
}

void Preferences::slotChanged()
{
    enableButtonApply(true);
}
