/***************************************************************************
*   Copyright (C) 2003 by Hideki Ikemoto                                  *
*   ikemo@wakaba.jp                                                       *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
***************************************************************************/

#include "threadlistheaderview.h"

#include <QtGui/QContextMenuEvent>

#include "kaction.h"
#include "klocale.h"
#include "kmenu.h"

#include "threadlistviewitem.h"

using namespace Kita;

const struct ColumnAttribute ThreadListHeaderView::attribute[] = {
    // labelName, itemName,
    // keyName, showDefault
    { " ", I18N_NOOP("Mark"),
        "ColumnMark", true },
    { I18N_NOOP2("@title:column", "No."), I18N_NOOP("ID"),
        "ColumnId", true },
    { " ", I18N_NOOP("Icon"),
        "ColumnIcon", true },
    { I18N_NOOP2("@title:column", "Title"), I18N_NOOP("Subject"),
        "ColumnSubject", true },
    { I18N_NOOP2("@title:column", "ResNum"), I18N_NOOP("ResNum"),
        "ColumnResNum", true },
    { I18N_NOOP2("@title:column", "ReadNum"), I18N_NOOP("ReadNum"),
        "ColumnReadNum", true },
    { I18N_NOOP2("@title:column", "ViewPos"), I18N_NOOP("ViewPos"),
        "ColumnViewPos", false },
    { I18N_NOOP2("@title:column", "Unread"), I18N_NOOP("Unread"),
        "ColumnUnread", true },
    { I18N_NOOP2("@title:column", "Since"), I18N_NOOP("Since"),
        "ColumnSince", true },
    { I18N_NOOP2("@title:column", "Thread's speed"),
        I18N_NOOP("Speed"), "ColumnSpeed", true },
    { I18N_NOOP2("@title:column", "Board"), I18N_NOOP("Board"),
        "ColumnBoard", false },
    { I18N_NOOP2("@title:column", "Dat URL"), I18N_NOOP("DatURL"),
        "ColumnDatUrl", false },
    { I18N_NOOP2("@title:column", "ID Order"), I18N_NOOP("IDOrder"),
        "ColumnIdOrder", false }
};

ThreadListHeaderView::ThreadListHeaderView(QWidget* parent)
        : QHeaderView(Qt::Horizontal, parent)
{
    setClickable(true);
    m_autoResizeAct
        = new KAction(i18nc("@action:inmenu", "Auto Resize"), this);
    m_autoResizeAct->setCheckable(true);
}

void ThreadListHeaderView::contextMenuEvent(QContextMenuEvent *event)
{
    KMenu popup;
    for (int i = ColumnBegin; i <= ColumnEnd; i++) {
        if (i != ColumnSubject && i != ColumnIdOrder) {
            KAction* action = new KAction(i18n(attribute[i].itemName), this);
            action->setCheckable(true);
            action->setChecked(!isSectionHidden(i));
            action->setData(QVariant(i));
            popup.addAction(action);
        }
    }
//  m_autoResizeAct->setChecked(autoResize()); TODO
    popup.addAction(m_autoResizeAct);

    QAction* action = popup.exec(event->globalPos());
    if (!action) {
        return;
    }
    if (action == m_autoResizeAct) {
        //setAutoResize(!action->isChecked());TODO
    } else if (action->isChecked()) {
        showSection(action->data().toInt());
    } else {
        hideSection(action->data().toInt());
    }
    //saveHeaderOnOff(); TODO
}
