/***************************************************************************
*   Copyright (C) 2003 by Hideki Ikemoto                                  *
*   ikemo@users.sourceforge.jp                                            *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
***************************************************************************/
#ifndef KITATHREADVIEW_H
#define KITATHREADVIEW_H

#include <QtGui/QWidget>

#include <kurl.h>
#include <dom/html_base.h>

class QFrame;
class QHBoxLayout;
class QSpacerItem;
class QToolButton;
class QVBoxLayout;

class KComboBox;
class KXMLGUIClient;

/* mode , m_viewmode uses them. */
enum {
    VIEWMODE_MAINVIEW,
};

namespace KIO
{
    class Job;
}

namespace Kita
{
    class HTMLPart;
    class ThreadTabWidget;
    class SubjectLabel;
   
    /**
     *
     * @author Hideki Ikemoto <ikemo@users.sourceforge.jp>
     **/

    class ThreadView : public QWidget
    {
        Q_OBJECT

        ThreadTabWidget* m_parent;

    public:
        ThreadView(ThreadTabWidget* parent);
        ~ThreadView();
        const KUrl threadUrl() const;
        const KUrl datUrl() const;
        const QString selectedText() const;

        void setup(const KUrl& datUrl, int mode);
        void showStatusBar(const QString& information);
        int getViewMode() { return m_viewmode; }

    public slots:
        virtual void setFocus();

        void showThread(const KUrl& datUrl, int num);
        void setFont(const QFont& font);
        void slotReloadButton(int jumpNum = 0);
        void slotStopLoading();
        void focusSearchCombo();
        void slotDeleteButtonClicked();
        void slotSearchNext();
        void slotSearchPrev();
        void slotGobackAnchor();
        void slotGotoHeader();
        void slotGotoFooter();

    protected slots:
        void slotDOMNodeActivated(const DOM::Node& node);

    private:
        void insertSearchCombo();
        void setSubjectLabel(const QString& boardName,
                const QString& threadName, const QString& boardUrl);
        void updateButton();

        ThreadView(const ThreadView&);
        ThreadView& operator=(const ThreadView&);

        int m_serverTime;
        KUrl m_datUrl;
        HTMLPart* m_threadPart;

        bool m_revsearch;
        int m_viewmode;
        int m_rescode;

        QToolButton* writeButton;
        KComboBox* searchCombo;
        QToolButton* highLightButton;
        QToolButton* bookmarkButton;
        QToolButton* reloadButton;
        KComboBox* gotoCombo;
        QToolButton* deleteButton;
        QToolButton* closeButton;
        QFrame* threadFrame;
        QVBoxLayout* threadViewBaseLayout;
        QHBoxLayout* layout2;
        QSpacerItem* spacer2;

    private slots:
        void slotSearchButton();
        void slotBookmarkButtonClicked(bool on);
        void slotWriteButtonClicked(const QString& resstr = QString());
        void slotComboActivated(int index);
        void slotUpdateInfo();
        void slotSearchPrivate(bool rev);
        void slotCloseButton();

    signals:
        void showThreadCompleted(); /* to ThreadPart */
    };
}

#endif
