/***************************************************************************
 *   Copyright (C) 2007 by Kita Developers                                 *
 *   ikemo@users.sourceforge.jp                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "viewmediator.h"

#include <krun.h>
#include <kio/netaccess.h>

#include "boardtabwidget.h"
#include "favoritelistview.h"
#include "mainwindow.h"
#include "threadtabwidget.h"
#include "writetabwidget.h"
#include "libkita/datmanager.h"

using namespace Kita;

ViewMediator* ViewMediator::instance = 0;

ViewMediator::ViewMediator()
{
}

ViewMediator::~ViewMediator()
{
}

ViewMediator* ViewMediator::getInstance()
{
    if(instance == 0) {
        instance = new ViewMediator();
    }
    return instance;
}

void ViewMediator::closeThreadTab(const KUrl& url)
{
    Q_ASSERT(m_threadTabWidget);

    m_threadTabWidget->slotCloseThreadTab(url);
}

void ViewMediator::showWriteView(const KUrl& url, const QString& resStr)
{
    Q_ASSERT(m_writeTabWidget);

    m_writeTabWidget->resize(500, 400);
    m_writeTabWidget->show();
    m_writeTabWidget->slotShowWriteView(url, resStr);
}

void ViewMediator::openBoard(const KUrl& url)
{
    Q_ASSERT(m_boardTabWidget);

    m_boardTabWidget->loadBoard(url);
}

void ViewMediator::openThread(const KUrl& url)
{
    Q_ASSERT(m_threadTabWidget);

    m_threadTabWidget->slotShowMainThread(url);
}

void ViewMediator::setMainStatus(const QString& statusStr)
{
    Q_ASSERT(m_mainWindow);

    m_mainWindow->setMainStatus(statusStr);
}

void ViewMediator::setMainUrlLine(const KUrl& url)
{
    Q_ASSERT(m_mainWindow);

    m_mainWindow->setUrl(url);
}

void ViewMediator::setMainCaption(const QString& str)
{
    Q_ASSERT(m_mainWindow);

    m_mainWindow->setCaption(str);
}

void ViewMediator::bookmark(const QString& datUrl, bool on)
{
    Q_ASSERT(m_mainWindow);

    m_mainWindow->bookmark(datUrl, on);
}

bool ViewMediator::isActive()
{
    Q_ASSERT(m_mainWindow);

    if(m_mainWindow->isActiveWindow()) return true;

    return false;
}

void ViewMediator::updateBoardView(const KUrl& datUrl)
{
    Q_ASSERT(m_boardTabWidget);

    m_boardTabWidget->updateBoardView(datUrl);
}

void ViewMediator::updateThreadView(const KUrl& datUrl)
{
    Q_ASSERT(m_threadTabWidget);

    m_threadTabWidget->slotUpdateThreadTab(datUrl);
}

void ViewMediator::changeWriteTab(const KUrl& datUrl)
{
    Q_ASSERT(m_writeTabWidget);

    m_writeTabWidget->slotChangeWriteTab(datUrl);
}

void ViewMediator::updateFavoriteListView()
{
    Q_ASSERT(m_favoriteListView);

    m_favoriteListView->refresh();
}

void ViewMediator::openUrl(const KUrl& url)
{
    // open thread.
    if (DatManager(url).isThreadRecorded()) {
        m_threadTabWidget->slotShowMainThread(url);
        return;
    }

    QString mimetype = KIO::NetAccess::mimetype(url , m_threadTabWidget);
    KRun::runUrl(url, mimetype, 0);
}
