/***************************************************************************
*   Copyright (C) 2004 by Kita Developers                                 *
*   ikemo@users.sourceforge.jp                                            *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
***************************************************************************/

#include "writetabwidget.h"

#include <QtGui/QClipboard>

#include <kaction.h>
#include <kactioncollection.h>
#include <kmessagebox.h>

#include "writeview.h"
#include "libkita/boarddatabase.h"
#include "libkita/datmanager.h"
#include "libkita/kita_misc.h"

using namespace Kita;

WriteTabWidget::WriteTabWidget(QWidget* parent) : TabWidgetBase(parent)
{
    setXMLFile("writetabwidgetui.rc");

    setupActions();
}


WriteTabWidget::~WriteTabWidget() {}


/* public slot */
void WriteTabWidget::slotShowWriteView(const KUrl& url,
        const QString& resStr)
{
    openWriteView(url, resStr, QString());
}


/* private */
void WriteTabWidget::openWriteView(const KUrl& url,
                                        const QString& resStr, const QString& subject)
{
    // TODO: machiBBS kakiko support.
    BoardDatabase db(url);
    if (db.type() == Board_MachiBBS) {
//        KMessageBox::sorry(this,
//                i18n("Can't write to machi BBS in this version."),
//                "<(_ _)>");
//        return ;
    }

    // view exists
    WriteView* view = findWriteView(url);
    if (view) {
        if (view->body().length()) {
            if (KMessageBox::warningYesNo(this,
                    i18n("Do you want to clear the text?"), "Kita")
                    == KMessageBox::No) return;
        }

        // clear
        view->setMessage(resStr);
        setCurrentWidget(view);
        return ;
    }

    // TODO: refactoring.
    // create new write view & add it to tab
    QString threadName;

    // write res
    WriteView* new_dlg;
    threadName = DatManager(url).threadName();
    new_dlg = new WriteView(this, url);
    new_dlg->setMessage(resStr);
    addTab(new_dlg, threadName);
    setCurrentWidget(new_dlg);
}

/* private */
WriteView* WriteTabWidget::findWriteView(const KUrl& url)
{
    KUrl datUrl = getDatUrl(url);
    if (datUrl.isEmpty()) return 0;

    int max = count();
    if (max == 0) return 0;

    for(int i=0; i < max; i++) {
        WriteView * view = isWriteView(widget (i));
        if (view) {
            if (view->datUrl() == datUrl) return view;
        }
    }

    return 0;
}


/* private */
WriteView* WriteTabWidget::isWriteView(QWidget* w)
{
    return qobject_cast<WriteView*>(w);
}



/* when thread view is focused, this slot is called    */
/* See also ThreadView::setFocus.                  */ /* private slot */
void WriteTabWidget::slotChangeWriteTab(const KUrl& url)
{
    WriteView * view;
    int max = count();
    if (max == 0) return ;

    // disable all ok buttons.
    for(int i=0; i < max ; i++) {
        view = isWriteView(widget(i));
        if (view) view->slotEnableWriting(false);
    }

    // show current url page.
    view = findWriteView(url);
    if (view) {
        if (currentWidget() != view) setCurrentWidget(view);
        view->slotEnableWriting(true);
    }
}


/* protected */ /* virtual */
void WriteTabWidget::deleteWidget(QWidget* w)
{
    WriteView * view = isWriteView(w);

    if (view == 0) return ;

    if (view->body().length()) {
        if (KMessageBox::warningYesNo(this,
                i18n("If you close this dialog, you will lose text.\n"
                "Do you want to close?"), "Kita") == KMessageBox::No) return;
    }

    TabWidgetBase::deleteWidget(w);
}


/*--------------------------------*/
/* WriteTabWidget actions */


/* private */
void WriteTabWidget::setupActions()
{
    KAction* quoteclip_action = actionCollection()->addAction("writeview_quoteclip");
    quoteclip_action->setText(i18n("quote clipboard"));
    quoteclip_action->setShortcut(Qt::Key_F2);
    connect(quoteclip_action, SIGNAL(triggered()), this, SLOT(slotQuoteClipboard()));
}


/* public slot */
void WriteTabWidget::slotQuoteClipboard()
{
    WriteView * view = isWriteView(currentWidget());
    if (view) {
        QClipboard * clipboard = QApplication::clipboard();
        QString str = clipboard->text(QClipboard::Selection);
        if (str.isEmpty()) str = clipboard->text(QClipboard::Clipboard);
        if (!str.isEmpty()) {

            QString msg = "\n> " + str.replace('\n', "\n> ") + '\n';
            view->insertMessage(msg);
        }
    }
}
