/***************************************************************************
*   Copyright (C) 2004 by Kita Developers                                 *
*   ikemo@users.sourceforge.jp                                            *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
***************************************************************************/
#ifndef KITAWRITEVIEW_H
#define KITAWRITEVIEW_H

#include <QtCore/QByteArray>
#include <QtGui/QWidget>

#include <kio/job.h>

#include "ui_writedialogbase.h"

/* result code */
enum{
    K2ch_Unknown,
    K2ch_True,
    K2ch_False,
    K2ch_Error,
    K2ch_Check,
    K2ch_Cookie
};

namespace Kita {
    class PreviewPart;
    class WriteTabWidget;

    /**
     * 
     * Hideki Ikemoto
     **/
    class WriteView : public QWidget, public ::Ui::WriteDialogBase
    {
        Q_OBJECT

        WriteTabWidget* m_parent;

        WriteView(const WriteView&);
        WriteView& operator=(const WriteView&);

    protected:

        QByteArray m_array;
        KUrl m_bbscgi;
        int m_bbstype;
        KUrl m_datUrl;
        QString m_mailswap;

        void initUI();
        bool checkFields();
        QString buildPostMessage();
        int getWriteResNum();
        void processPostFinished();
        void logPostMessage();
        int resultCode(const QString& response) const;
        QString resultMessage(const QString& response) const;
        QString resultTitle(const QString& response) const;

    public:
        WriteView(WriteTabWidget* parent, const KUrl& url);
        virtual ~WriteView();
        void setMessage(const QString& bodyStr);
        void insertMessage(const QString& str);

        const KUrl datUrl() const;
        const QString threadName() const;
        const QString boardName() const;
        const QString boardId() const;
        const QString name() const
        {
          return nameLine->text();
        }

        const QString mail() const
        {
          return mailLine->text();
        }

        const QString body() const
        {
          return bodyText->toPlainText();
        }

    public slots:

        virtual void setFocus();
        void slotPostMessage();
        void slotCancel();
        void slotEnableWriting(bool enable);
        void sageBoxToggled(bool on)
        {
          if (on) {
            m_mailswap = mailLine->text();
            mailLine->setText("sage");
            mailLine->setReadOnly(true);
          } else {
            mailLine->setReadOnly(false);
            mailLine->setText(m_mailswap);
          }
        }

    private slots:

        void slotRecieveData(KIO::Job*, const QByteArray&);
        void slotPostFinished(KJob*);

    protected slots:
        bool slotBodyTextChanged();

    };
}
#endif
